@@ -140,24 +140,18 @@ class Module(object):
         ]:
             raise DriftError(f"Invalid mode {mode!r}.")
 
-        if channels is not None:
-            self.channels = channels
-        else:
-            if default_channels is None:
-                raise DriftError("Cannot determine module number of channels.")
-            self.channels = default_channels
-
+        self.channels = channels or default_channels
         self.devices = CaseInsensitiveDict()
 
         log.info(
             f"Created module {self.name} with mode {self.mode!r} "
-            f"and {self.channels} channels."
+            f"and {self.channels or '?'} channels."
         )
 
     def __repr__(self):
         return (
             f"<Module {self.name} (mode={self.mode!r}, "
-            f"channels={self.channels}, n_devices={len(self.devices)})>"
+            f"channels={self.channels or '?'}, n_devices={len(self.devices)})>"
         )
 
     def __getitem__(self, name: str) -> Device:
@@ -323,7 +317,7 @@ class Device(object):
         ]:
             raise DriftError(f"Invalid mode {mode!r}.")
 
-        if self.channel is not None:
+        if self.channel is not None and self.module.channels:
             assert self.channel < self.module.channels
 
         log.info(
@@ -595,18 +589,21 @@ class Drift(object):
         The IP of the TCP modbus server.
     port
         The port of the TCP modbus server.
+    lock
+        Whether to lock the client while it's in use, preventing
+        multiple connections. Only used with the contextual manager.
 
     """
 
-    def __init__(self, address: str, port: int = 502):
+    def __init__(self, address: str, port: int = 502, lock: bool = True):
 
         self.address = address
         self.port = port
         self.client = AsyncioModbusTcpClient(address, port=port)
 
-        self.modules = CaseInsensitiveDict()
+        self.lock = asyncio.Lock() if lock else None
 
-        self.lock = asyncio.Lock()
+        self.modules = CaseInsensitiveDict()
 
     def __repr__(self):
         return f"<Drift @ {self.address}>"
@@ -614,16 +611,19 @@ class Drift(object):
     async def __aenter__(self):
         """Initialises the connection to the server."""
 
-        await self.lock.acquire()
+        if self.lock:
+            await self.lock.acquire()
 
         try:
             await asyncio.wait_for(self.client.connect(), timeout=1)
         except asyncio.TimeoutError:
-            self.lock.release()
+            if self.lock:
+                self.lock.release()
             raise DriftError(f"Failed connecting to server at {self.address}.")
 
         if not self.client.connected:
-            self.lock.release()
+            if self.lock:
+                self.lock.release()
             raise DriftError(f"Failed connecting to server at {self.address}.")
 
         log.debug(f"Connected to {self.address}.")
@@ -635,15 +635,28 @@ class Drift(object):
 
         self.client.stop()
         log.debug(f"Disonnected from {self.address}.")
-        self.lock.release()
+
+        if self.lock:
+            self.lock.release()
 
         return
 
-    def __getitem__(self, name: str):
+    def __getitem__(self, name: str) -> Module:
         """Gets a module."""
 
         return self.modules[name]
 
+    @property
+    def devices(self):
+        """Lists the devices connected across all modules."""
+
+        devices = []
+        for module in self.modules:
+            for device in self[module].devices:
+                devices.append(device)
+
+        return devices
+
     def add_module(self, name: str, **kwargs) -> Module:
         """Adds a new module.
 
@@ -701,6 +714,8 @@ class Drift(object):
 
         Parameters
         ----------
+        name
+            The device to read.
         adapt
             If possible, convert the value to real units.
 
@@ -713,6 +728,53 @@ class Drift(object):
 
         return await self.get_device(name).read(adapt=adapt)
 
+    async def read_devices(
+        self,
+        devices: list[str],
+        adapt: bool = True,
+        lock: bool | None = None,
+    ):
+        """Reads a list of devices minimising the number of connections to the client.
+
+        Parameters
+        ----------
+        devices
+            The list of devices to read.
+        adapt
+            If possible, convert the value to real units.
+        lock
+            If `True`, lock the client while reading the devices. If `None`, use
+            the default locking mechanism.
+
+        Returns
+        -------
+        read_values
+            A list of read values. If `adapt=True`, each element is a tuple of
+            value and unit.
+
+        """
+
+        lock = lock if lock is not None else (self.lock is not None)
+
+        tasks = []
+        for device in devices:
+            tasks.append(self.get_device(device).read(adapt=adapt, connect=False))
+
+        if lock and self.lock:
+            await self.lock.acquire()
+
+        try:
+            await asyncio.wait_for(self.client.connect(), timeout=1)
+            results = await asyncio.gather(*tasks)
+        finally:
+            if self.client.connected:
+                self.client.stop()
+
+            if lock and self.lock:
+                self.lock.release()
+
+        return results
+
     async def read(self, *args, **kwargs):
         """Alias for `.read_device`."""
 
